# -*- coding: utf-8 -*-

import pytest
import os
import shutil

from video_compressor.exceptions import MissingLibraryError, InvalidVideoInput
import video_compressor as vp

__author__ = "Lenselle Nicolas"
__copyright__ = "Lenselle Nicolas"
__license__ = "mit"

# remove sound on video
# check if video has sound or not


@pytest.fixture(scope="function")
def tempdir():
    return os.getcwd() + '/tmp/'


@pytest.fixture(scope="function")
def temp(tempdir):
    def path(filename=''):
        return tempdir + filename
    return path


@pytest.fixture(scope="function", autouse=True)
def beforeAll(tempdir):
    if os.path.exists(tempdir):
        shutil.rmtree(tempdir)
    os.mkdir(tempdir)
    yield
    shutil.rmtree(tempdir)


def VideoInfo(input, **options):
    return vp.VideoInfo(
        input,
        mp4info_bin='/Users/nicolaslenselle/Downloads/Bento4-SDK-1-6-0-637.universal-apple-macosx/bin/mp4info',
        **options
    )

def VideoCompressor(input, **options):
    return vp.VideoCompressor(
        input,
        mp4info_bin='/Users/nicolaslenselle/Downloads/Bento4-SDK-1-6-0-637.universal-apple-macosx/bin/mp4info',
        mp4fragment_bin='/Users/nicolaslenselle/Downloads/Bento4-SDK-1-6-0-637.universal-apple-macosx/bin/mp4fragment',
        **options
    )

def test_error_with_invalid_ffmpeg():
    VideoInfo('./tests/sample.mp4', ffprobe_bin='ffprobe')
    with pytest.raises(MissingLibraryError):
        VideoInfo('./tests/sample.mp4', ffprobe_bin='ffprobe-custom-bin')


def test_error_with_invalid_input():

    with pytest.raises(InvalidVideoInput):
        VideoInfo('./tests/corrupted-sample.mp4')

    with pytest.raises(InvalidVideoInput):
        VideoInfo('./tests/unexisting-sample.mp4')


def test_error_with_invalid_ouput():
    pass


def test_error_with_invalid_mute_value():
    pass


def test_error_with_invalid_scale_value():
    pass


def test_get_video_audio():
    assert VideoInfo('./tests/sample.mp4').volumedetect() is True
    assert VideoInfo('./tests/mute-sample.mp4').volumedetect() is False


def test_getVideoBitrate():
    assert VideoInfo('./tests/sample.mp4').getVideoBitrate() == 2200634


def test_getAudioBitrate():
    assert VideoInfo('./tests/sample.mp4').getAudioBitrate() == 133274


def test_get_video_resolution():
    video = VideoInfo('./tests/sample.mp4')
    w, h = video.getResolution()
    assert h == 540
    assert w == 960


def test_get_video_fps():
    video = VideoInfo('./tests/sample.mp4')
    fps = video.getFramePerSeconds()
    assert fps == 30

def test_get_video_duration_():
    video = VideoInfo('./tests/sample.mp4')
    d = video.getDurationInMicroseconds()
    assert d == 4_871_533

def test_get_video_duration_in_seconds():
    video = VideoInfo('./tests/sample.mp4')
    d = video.getDurationInSeconds()
    assert d == 5

def test_get_video_size():
    video = VideoInfo('./tests/sample.mp4')
    s = video.getSize()
    assert s == 1507453

def test_check_if_video_fragmentation(temp):
    video = VideoInfo('./tests/sample.mp4')
    fragmented = video.isFragmented()
    assert fragmented is False

def test_mute_a_video(temp):
    video = VideoCompressor(input='./tests/sample.mp4')

    sample_copy = temp(filename='sample-copy.mp4')
    sample_muted = temp(filename='sample-muted.mp4')

    video.export(sample_copy)
    video.mute(True).export(sample_muted)

    assert VideoInfo(sample_copy).volumedetect() is True
    assert VideoInfo(sample_muted).volumedetect() is False


def test_scale_video(temp):

    sample54x96 = temp(filename='sample-54x96.mp4')

    w, h = VideoInfo('./tests/sample.mp4').getResolution()
    assert w == 960
    assert h == 540

    video = VideoCompressor(input='./tests/sample.mp4')
    video.scale(96, 54).export(sample54x96)
    w, h = VideoInfo(sample54x96).getResolution()
    assert w == 96
    assert h == 54

def test_reduce_video_fps(temp):

    sample24fps = temp(filename='sample-24fps.mp4')
    video = VideoCompressor(input='./tests/sample.mp4')
    video.fps(24).export(sample24fps)

    fps = VideoInfo(sample24fps).getFramePerSeconds()
    assert fps == 24


def test_scale_video_keep_ratio(temp):

    sample640 = temp(filename='sample-54x96.mp4')

    w1, h1 = VideoInfo('./tests/sample.mp4').getResolution()

    video = VideoCompressor(input='./tests/sample.mp4')
    video.scale(640, -1).export(sample640)
    w2, h2 = VideoInfo(sample640).getResolution()
    assert w2 // h2 == w1 // h1


def test_reduce_video_bitrate(temp):

    sample1Mbs = temp(filename='sample-1mbs.mp4')
    video = VideoCompressor(input='./tests/sample.mp4')
    video.bitrate(1_000_000).export(sample1Mbs)

    assert VideoInfo(sample1Mbs).getVideoBitrate() < 1_000_000


def test_crop_video(temp):

    sampleCrop = temp(filename='sample-crop.mp4')
    video = VideoCompressor(input='./tests/sample.mp4')
    video.crop(origin=(10, 10), size=(100, 200)).export(sampleCrop)

    w, h = VideoInfo(sampleCrop).getResolution()
    assert w == 100
    assert h == 200

def test_combine_filter_crop_fps_bitrate(temp):

    Crop24fps1kBitrate = temp(filename='sample-crop-24ps-1kbitrate.mp4')
    video = VideoCompressor(input='./tests/sample.mp4')
    
    (video
        .crop(origin=(10, 10), size=(100, 200))
        .fps(24)
        .bitrate('2M')
        .mute(True)
        .export(Crop24fps1kBitrate)
    )

    info = VideoInfo(Crop24fps1kBitrate)
    w, h = video.info.getResolution()
    fps = video.info.getFramePerSeconds()
    
    assert list(info.getResolution()) == [100, 200]
    assert info.getFramePerSeconds() == 24
    assert info.getVideoBitrate() < 2_000_000

def test_combine_filter_scale_fps_bitrate_mute(temp):

    Crop24fps1kBitrate = temp(filename='sample-crop-24ps-1kbitrate.mp4')
    video = VideoCompressor(input='./tests/sample.mp4')
    
    (video
        .scale(640)
        .fps(24)
        .bitrate(1_000_000)
        .mute(True)
        .export(Crop24fps1kBitrate)
    )

    info = VideoInfo(Crop24fps1kBitrate)
    w, h = video.info.getResolution()
    fps = video.info.getFramePerSeconds()
    
    assert list(info.getResolution())[0] == 640
    assert info.getFramePerSeconds() == 24
    assert info.getVideoBitrate() < 1_000_000
    assert info.volumedetect() is False


def test_crop_video_checking_pixel(temp):
    # Test to compare pixel value from original video and crop video
    pass


def test_slice_video_by_1_seconds(temp):
    
    video = VideoCompressor(input='./tests/sample.mp4')

    slices = video.slice(temp('sample-slice.mp4'), stepInMilliseconds=1000)
    assert slices.getDurationInMicroseconds() == video.info.getDurationInMicroseconds()

    slices = video.slice(temp('sample-slice025.mp4'), stepInMilliseconds=2000)
    assert slices.getDurationInMicroseconds() == video.info.getDurationInMicroseconds()

def test_export_video_collection(temp):

    settings = [
        {'codec_preset': 'h264WebVBR', 'scale':[480, -1], 'bitrate': 200_000, 'fps': 24, 'suffix':'@sm'},
        {'codec_preset': 'h264WebVBR', 'scale':[480, -1], 'bitrate': 200_000, 'fps': 24, 'quality':'low', 'suffix':'@sm+low'},
        {'scale':[640, -1], 'bitrate': 1_000_000, 'fps': 24, 'suffix':'@md'},
        {'scale':[960, -1], 'bitrate': 2_000_000, 'fps': 24, 'suffix':'@lg'},
        {'scale':[1280, -1], 'bitrate': 3_000_000, 'fps': 24, 'suffix':'@xl'},
    ]

    video = VideoCompressor('./tests/sample.mp4')
    list(video.exportCollection(temp('sample.mp4'), settings))

    for setting in settings:
        suffix = setting['suffix']
        export = VideoInfo(temp(f'sample{suffix}.mp4'))
        assert list(export.getResolution())[0] == setting['scale'][0]
        assert export.getFramePerSeconds() == setting['fps']
        assert export.getVideoBitrate() < setting['bitrate']

    assert VideoInfo(temp(f'sample@sm+low.mp4')).getSize() < VideoInfo(temp(f'sample@sm.mp4')).getSize()


def test_hs264_webpreset(temp):

    h264WebPreset = temp('webh264.mp4')
    video = VideoCompressor('./tests/sample.mp4')
    video.bitrate('1M').codecPreset('h264WebVBR').export(h264WebPreset)

    assert VideoInfo(h264WebPreset).getSize() < video.info.getSize()


def test_export_video_with_height_not_divisable_by_two(temp):

    video = VideoCompressor(input='./tests/sample.mp4')

    w, h = video.info.getResolution()
    video.crop(origin=(0, 0), size=(w, h-1)).export(temp('video-crop.mp4'))

    videoCrop = VideoCompressor(input=temp('video-crop.mp4'))
    videoCrop.scale(480, -1).export(temp('video-crop2.mp4'))
    

def test_fragment_a_video(temp):

    video = VideoCompressor('./tests/sample.mp4')
    assert video.info.isFragmented() is False

    sample_fragmented = temp(filename='sample-fragment.mp4')

    video.fragment(sample_fragmented)
    assert VideoInfo(sample_fragmented).isFragmented() is True

# targetSize=$(( 25 * 1000 * 1000 * 8 )) # 25MB in bits
# length=`ffprobe -v error -show_entries format=duration -of default=noprint_wrappers=1:nokey=1 input.mp4`
# length_round_up=$(( ${length%.*} + 1 ))
# total_bitrate=$(( $targetSize / $length_round_up ))
# audio_bitrate=$(( 128 * 1000 )) # 128k bit rate
# video_bitrate=$(( $total_bitrate - $audio_bitrate ))
# ffmpeg -i input.mp4 -b:v $video_bitrate -maxrate:v $video_bitrate
# -bufsize:v $(( $targetSize / 20 )) -b:a $audio_bitrate output.mp4



from video_compressor import functions 


def test_compute_slice_video_frames():

    assert list(functions.rangeSliceBySteps(0, 1, step=1)) == [(0, 1)]
    assert list(functions.rangeSliceBySteps(0, 2, step=1)) == [(0, 1), (1, 1)]
    assert list(functions.rangeSliceBySteps(0, 3, step=1)) == [(0, 1), (1, 1), (2, 1)]

    assert list(functions.rangeSliceBySteps(0, 1, step=2)) == [(0, 1)]
    assert list(functions.rangeSliceBySteps(0, 8, step=2)) == [(0, 2), (2, 2), (4, 2), (6, 2)]
    assert list(functions.rangeSliceBySteps(0, 9, step=2)) == [(0, 2), (2, 2), (4, 2), (6, 2), (8, 1)]

    assert list(functions.rangeSliceBySteps(1, 1, step=1)) == []
    assert list(functions.rangeSliceBySteps(1, 3, step=1)) == [(1, 1), (2, 1)]
    assert list(functions.rangeSliceBySteps(3, 8, step=2)) == [(3, 2), (5, 2), (7, 1)]


import os
from video_compressor.adapters.ffmpeg import ffmpegVideoCompressorAdapter, ffmpegProbeVideoInfoAdapter
import video_compressor.functions as vfunctions

class VideoInfo():

    @classmethod
    def defaultInfoAdapter(cls):
        return ffmpegProbeVideoInfoAdapter

    def __init__(self, path, adapter=None, **options):
        self.path = path
        self.adapter = adapter or VideoInfo.defaultInfoAdapter()(input=path, **options)
        self.adapter.check_video_integrity(path)

    def volumedetect(self):
        return self.adapter.volumedetect()

    def getVideoBitrate(self):
        return self.adapter.getVideoBitrate()

    def getAudioBitrate(self):
        return self.adapter.getAudioBitrate()

    def getResolution(self):
        return self.adapter.getResolution()

    def getDurationInMicroseconds(self):
        return self.adapter.getDurationInMicroseconds()

    def getDurationInMilliseconds(self):
        microseconds = self.getDurationInMicroseconds()
        return round(microseconds/1000)

    def getDurationInSeconds(self):
        milliseconds = self.getDurationInMilliseconds()
        return round(milliseconds/1000)

    def getSize(self):
        return self.adapter.getSize()

    def getFramePerSeconds(self):
        return self.adapter.getFramePerSeconds()

    def isFragmented(self):
        return self.adapter.isFragmented()
class VideoInfoCollection():

    def __init__(self, videos=None):
        self._videos = [VideoInfo(video) for video in (videos or [])]

    def __len__(self):
        return len(self._videos)

    def append(self, video, adapter_options={}):
        self._videos.append(VideoInfo(video, **adapter_options))

    def getDurationInMicroseconds(self):
        return sum(map(lambda s: s.getDurationInMicroseconds(), self._videos))


class VideoCompressor():

    WebSettings = [
        {'scale':[480, -1], 'bitrate': '200k', 'fps': 24, 'suffix':'@sm'},
        {'scale':[640, -1], 'bitrate': '1M', 'fps': 24, 'suffix':'@md'},
        {'scale':[960, -1], 'bitrate': '2M', 'fps': 24, 'suffix':'@lg'},
        {'scale':[1280, -1], 'bitrate': '3M', 'fps': 24, 'suffix':'@xl'},
    ]

    @classmethod
    def defaultCompressorAdapter(cls):
        return ffmpegVideoCompressorAdapter

    def __init__(
        self,
        input=None,
        mute=None,
        scale=None,
        bitrate=None,
        crop_origin=None,
        crop_size=None,
        fps=None,
        codec_preset=None,
        quality=None,
        suffix="",
        adapter=None,
        **adapter_options,
    ):
        self._input = input
        self._mute = mute
        self._scale = scale
        self._bitrate = bitrate
        self._crop_origin = crop_origin
        self._crop_size = crop_size
        self._fps = fps
        self._codec_preset = codec_preset
        self._quality = quality
        self._suffix = suffix
        self._adapter_options = adapter_options

        self.VideoCompressorAdapter = adapter or VideoCompressor.defaultCompressorAdapter()

    @property
    def compressor_adapter(self):
        return self.VideoCompressorAdapter(
            input=self._input,
            mute=self._mute,
            scale=self._scale,
            bitrate=self._bitrate,
            crop_origin=self._crop_origin,
            crop_size=self._crop_size,
            codec_preset=self._codec_preset,
            quality=self._quality,
            fps=self._fps,
            **self._adapter_options
        )

    @property
    def info(self):
        return VideoInfo(self._input, **self._adapter_options)

    def options(self):
        return {
            'input': self._input,
            'mute': self._mute,
            'scale': self._scale,
            'bitrate': self._bitrate,
            'crop_origin': self._crop_origin,
            'crop_size': self._crop_size,
            'codec_preset': self._codec_preset,
            'quality': self._quality,
            'fps': self._fps,
        }

    def update(self, **updates):
        options = self.options()
        options.update(updates)
        video = VideoCompressor(**options, **self._adapter_options)
        video.VideoCompressorAdapter = self.VideoCompressorAdapter
        return video

    def mute(self, mute):
        return self.update(mute=mute)

    def scale(self, *scale):
        if len(scale) == 1:
            return self.update(scale=(scale[0], -1))
        return self.update(scale=scale)

    def bitrate(self, bitrate):
        return self.update(bitrate=bitrate)

    def crop(self, origin, size):
        return self.update(crop_origin=origin, crop_size=size)

    def fps(self, fps):
        return self.update(fps=fps)

    def codecPreset(self, codec_preset):
        return self.update(codec_preset=codec_preset)

    def quality(self, quality):
        return self.update(quality=quality)

    def export(self, output):
        filename, ext = os.path.splitext(output)
        return self.compressor_adapter.export(f'{filename}{self._suffix}{ext}')

    def fragment(self, output):
        return self.compressor_adapter.fragment(output)

    def slice(self, output, stepInMilliseconds=1000):
        
        videos = VideoInfoCollection()
        path, ext = os.path.splitext(output)
        durationInMilliseconds = self.info.getDurationInMilliseconds()

        steps = vfunctions.rangeSliceBySteps(
            0, 
            durationInMilliseconds, 
            stepInMilliseconds
        )

        for i, step in enumerate(steps):
            start, duration = step
            step_path = f'{path}-{i}{ext}'
            self.compressor_adapter.slice(step_path, start, duration)
            videos.append(step_path, self._adapter_options)

        return videos

    def exportCollection(self, output, settings):
        for setting in settings:
            video = self.update(**setting)
            video.export(output)
            yield video



from video_compressor import VideoCompressor

input = VideoCompressor(input='/Users/nicolaslenselle/Downloads/pexel.mp4')
input.exportCollection('./pexel.mp4', input.WebSettings)
input.codecPreset('h264WebVBR').exportCollection('./withcodec.mp4', input.WebSettings)


from itertools import chain

def rangeSliceBySteps(start, stop, step=1):

    length = stop - start
    congruent_stop = start + (length - (length % step))

    for step_slice in map(lambda i: (i, step), range(start, congruent_stop, step)):
        yield step_slice

    if congruent_stop < stop:
        yield (congruent_stop, stop - congruent_stop)




from shutil import which
import subprocess
import os
import json

from ..exceptions import MissingLibraryError, InvalidVideoInput


def process(command):
    process = subprocess.run(command, shell=True, stdout=subprocess.PIPE)
    return (
        (process.stdout or b'').decode("utf-8"),
        (process.stderr or b'').decode("utf-8")
    )


def check_bin(bin):
    if which(bin) is None:
        raise MissingLibraryError
    return bin


class ffprobeCmdBuilder():

    def __init__(
        self,
        input,
        bin='ffprobe'
    ):
        self.input = input
        self.bin = check_bin(bin)

    @property
    def ffprobe(self):
        return f"{self.bin} -v error -select_streams v:0 -of csv=s=,:p=0"

    @property
    def resolution(self):
        return f"{self.ffprobe} -show_entries stream=width,height {self.input}"

    @property
    def video_bitrate(self):
        return f"{self.ffprobe} -show_entries stream=bit_rate {self.input}"

    @property
    def audio_bitrate(self):
        return f"{self.ffprobe} -select_streams a:0 -show_entries stream=bit_rate {self.input}"

    @property
    def duration(self):
        return f"{self.ffprobe} -show_entries stream=duration {self.input}"

    @property
    def fps(self):
        return f"{self.ffprobe} -v error -select_streams v -of default=noprint_wrappers=1:nokey=1 -show_entries stream=r_frame_rate {self.input}"



class ffmpegCmdBuilder():

    def __init__(
        self,
        input=None,
        bin=None,
        mute=None,
        scale=None,
        bitrate=None,
        crop_origin=None,
        crop_size=None,
        codec_preset=None,
        codec_pass=None,
        quality=None,
        fps=None
    ):
        self.input = input
        self.bin = check_bin(bin)
        self.mute = mute
        self.scale = scale
        self.bitrate = bitrate
        self.crop_origin = crop_origin
        self.crop_size = crop_size
        self.codec_preset = codec_preset
        self.codec_pass = codec_pass
        self.quality = quality
        self.fps = fps

    @property
    def ffmpeg(self):
        return f"{self.bin} -i {self.input}"

    @property
    def mutefilter(self):
        return '-an' if self.mute else ''

    @property
    def scalefilter(self):

        if not self.scale:
            return ""

        w, h = self.scale
        if w == -1:
            w = 'trunc(oh/a/2)*2'
        if h == -1:
            h = 'trunc(ow/a/2)*2'

        return f'scale={w}:{h}'

    @property
    def bitratefilter(self):
        return f'-b:v {self.bitrate}' if self.bitrate else ''

    @property
    def fpsfilter(self):
        return f'fps=fps={self.fps}' if self.fps else ''

    @property
    def cropfilter(self):
        if not self.crop_origin and not self.crop_size:
            return ""

        x, y = self.crop_origin
        w, h = self.crop_size 

        return f'crop={w}:{h}:{x}:{y}'

    @property
    def pipestdout(self):
        return '-f null /dev/null 2>&1'

    @property
    def integrity(self):
        return f'{self.ffmpeg} -v error {self.pipestdout}'

    @property
    def volumedetect(self):
        return f"{self.ffmpeg} -af 'volumedetect' {self.pipestdout}"

    @property
    def vfilters(self):
        filters = list(filter(lambda f: f != '', [
            self.cropfilter,
            self.fpsfilter,
            self.scalefilter
        ]))

        if len(filters) > 0:
            return '-vf "'+ ','.join(filters) + '"'
        else:
            return ""

    @property
    def filters(self):
        return f'{self.mutefilter} {self.bitratefilter} {self.vfilters}'

    @property
    def crf_quality(self):
        return ({
            'low': '-crf 35',
            'max': '-crf 0' 
        }).get(self.quality, '-crf 24')

    @property
    def codec(self):
        if self.codec_preset == 'h264WebVBR':
            return f"-c:v libx264 {self.crf_quality} -profile:v main -level 4.0"
        else:
            return ""

    def export(self, output):
        return f'{self.ffmpeg} {self.codec} {self.filters} {output}'

    def slice(self, output, start, duration):
        return f'{self.ffmpeg} {self.codec} {self.filters} -ss {start} -t {duration} {output}'


class mp4InfoCmdBuilder():

    def __init__(self, bin, input):
        self._bin = check_bin(bin)
        self._input = input

    @property
    def info(self):
        return f"{self._bin} --format json {self._input}"


class ffmpegProbeVideoInfoAdapter():

    def __init__(self, input=None, ffmpeg_bin='ffmpeg', ffprobe_bin='ffprobe', mp4info_bin='mp4info', **kwargs):
        self.input = input
        self.ffprobe = ffprobeCmdBuilder(input=input, bin=ffprobe_bin)
        self.ffmpeg = ffmpegCmdBuilder(input=input, bin=ffmpeg_bin)
        self.mp4Info = mp4InfoCmdBuilder(input=input, bin=mp4info_bin)

    def check_video_integrity(self, input):
        trace, error = process(self.ffmpeg.integrity)
        if 'Invalid data' in trace or 'No such file or directory' in trace:
            raise InvalidVideoInput(trace)

    def volumedetect(self):
        volumetrace, error = process(self.ffmpeg.volumedetect)
        return ('Parsed_volumedetect' in volumetrace)

    def getResolution(self):
        resolution, error = process(self.ffprobe.resolution)
        return map(int, resolution.split(','))

    def getVideoBitrate(self):
        bitrate, error = process(self.ffprobe.video_bitrate)
        return int(bitrate) if bitrate else 0

    def getAudioBitrate(self):
        bitrate, error = process(self.ffprobe.audio_bitrate)
        return int(bitrate) if bitrate else 0

    def getDurationInMicroseconds(self):
        duration, error = process(self.ffprobe.duration)
        duration = duration.replace('\n', '')
        return round(float(duration) * 1000 * 1000)

    def getSize(self):
        return os.path.getsize(self.input)

    def getFramePerSeconds(self):
        rate, error = process(self.ffprobe.fps)
        return round(eval(rate))

    def isFragmented(self):
        info, error = process(self.mp4Info.info)
        info = json.loads(info)
        return info['movie']['fragments']
class ffmpegVideoCompressorAdapter():

    def __init__(
        self,
        input=None,
        ffmpeg_bin='ffmpeg',
        ffprobe_bin='ffprobe',
        mp4info_bin='mp4info',
        mp4fragment_bin='mp4fragment',
        mute=None,
        scale=None,
        bitrate=None,
        crop_origin=None,
        crop_size=None,
        codec_preset=None,
        quality=None,
        fps=None,
    ):
        self._bin_ffmpeg = ffmpeg_bin
        self._bin_ffprobe = ffprobe_bin
        self._bin_mp4info = mp4info_bin
        self._bin_mp4fragment = mp4fragment_bin
        self._input = input
        self._mute = mute
        self._scale = scale
        self._bitrate = bitrate
        self._crop_origin = crop_origin
        self._crop_size = crop_size
        self._fps = fps
        self._codec_preset = codec_preset
        self._quality = quality

    @property
    def ffmpeg(self):
        return ffmpegCmdBuilder(**{
            'input': self._input,
            'bin': self._bin_ffmpeg,
            'mute': self._mute,
            'scale': self._scale,
            'bitrate': self._bitrate,
            'crop_origin': self._crop_origin,
            'crop_size': self._crop_size,
            'codec_preset': self._codec_preset,
            'quality': self._quality,
            'fps': self._fps
        })

    def export(self, output):
        process(self.ffmpeg.export(output))

    def slice(self, output, startInMillisecond, durationInMicroseconds):
        start_str = str(round(startInMillisecond / 1000, 3))
        duration_str = str(round(durationInMicroseconds / 1000, 3))
        process(self.ffmpeg.slice(output, start_str, duration_str))

    def fragment(self, output):
        check_bin(self._bin_mp4fragment)
        process(f'{self._bin_mp4fragment} {self._input} {output}')